import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import {
  createPersonalOffersDtoDataMock,
  updatePersonalOffersDtoDataMock,
  personalOffersDataMock,
} from '../../test/mock/data/personalOffers.mock.data';
import { PersonalOffersService } from './personal-offers.service';

describe('PersonalOffersService', () => {
  let service: PersonalOffersService;

  const personalOfferModelMock = {
    findById: jest.fn(),
    create: jest.fn(),
    findByIdAndUpdate: jest.fn(),
    findByIdAndDelete: jest.fn(),
    exec: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        PersonalOffersService,
        {
          provide: getModelToken('PersonalOffer'),
          useValue: personalOfferModelMock,
        },
      ],
    }).compile();

    service = module.get<PersonalOffersService>(PersonalOffersService);
  });

  afterEach(async () => {
    jest.clearAllMocks();
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('findOne', () => {
    it('should return personal offer', async () => {
      personalOfferModelMock.findById.mockReturnThis();
      personalOfferModelMock.exec.mockResolvedValueOnce(personalOffersDataMock[0]);
      expect(await service.findOne('1234ba0e2ab5775cfc01ed3e')).toEqual(personalOffersDataMock[0]);
    });
    it('should return exception if personal offer is not found for given id', async () => {
      personalOfferModelMock.findById.mockReturnThis();
      personalOfferModelMock.exec.mockResolvedValueOnce(null);
      let error: any;
      try {
        await service.findOne('abcd');
      } catch (e) {
        error = e;
      }
      expect(error.message).toBe('Personal offer does not exist');
      expect(error.status).toBe(404);
    });
  });

  describe('create', () => {
    it('should create personal offer', async () => {
      personalOfferModelMock.create.mockResolvedValueOnce(personalOffersDataMock[0]);
      expect(await service.create(createPersonalOffersDtoDataMock[0])).toEqual(personalOffersDataMock[0]);
    });
  });

  describe('update', () => {
    it('should update personal offer', async () => {
      personalOfferModelMock.findByIdAndUpdate.mockReturnThis();
      personalOfferModelMock.exec.mockResolvedValueOnce(personalOffersDataMock[1]);
      expect(await service.update('2345ba0e2ab5775cfc01ed4d', updatePersonalOffersDtoDataMock[1])).toEqual(
        personalOffersDataMock[1]
      );
    });
    it('should return exception if personal offer is not found for given id', async () => {
      personalOfferModelMock.findById.mockReturnThis();
      personalOfferModelMock.exec.mockResolvedValueOnce(null);
      let error: any;
      try {
        await service.update('abcd', updatePersonalOffersDtoDataMock[1]);
      } catch (e) {
        error = e;
      }
      expect(error.message).toBe('Invalid personal offer id for update');
      expect(error.status).toBe(400);
    });
  });

  describe('delete', () => {
    it('should update personal offer', async () => {
      personalOfferModelMock.findByIdAndDelete.mockReturnThis();
      personalOfferModelMock.exec.mockResolvedValueOnce(personalOffersDataMock[0]);
      expect(await service.delete('2345ba0e2ab5775cfc01ed4d')).toEqual(personalOffersDataMock[0]);
    });
    it('should return exception if personal offer is not found for given id', async () => {
      personalOfferModelMock.findById.mockReturnThis();
      personalOfferModelMock.exec.mockResolvedValueOnce(null);
      let error: any;
      try {
        await service.delete('abcd');
      } catch (e) {
        error = e;
      }
      expect(error.message).toBe('Invalid personal offer id for deletion');
      expect(error.status).toBe(400);
    });
  });
});
