import { HttpModule } from '@nestjs/common';
import { ConfigModule } from '@nestjs/config';
import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import * as bcrypt from 'bcrypt';
import { Types } from 'mongoose';
import { personalOffersDataMock } from '../../../test/mock/data/personalOffers.mock.data';
import { structuresDocumentDataMock } from '../../../test/mock/data/structures.mock.data';
import { mockParametersModel } from '../../../test/mock/services/parameters.mock.service';
import { UsersServiceMock } from '../../../test/mock/services/user.mock.service';
import { CategoriesFormationsService } from '../../categories/services/categories-formations.service';
import { ConfigurationService } from '../../configuration/configuration.service';
import { MailerModule } from '../../mailer/mailer.module';
import { ParametersService } from '../../parameters/parameters.service';
import { Parameters } from '../../parameters/schemas/parameters.schema';
import { PersonalOfferDocument } from '../../personal-offers/schemas/personal-offer.schema';
import { SearchModule } from '../../search/search.module';
import { IUser } from '../../users/interfaces/user.interface';
import { UsersService } from '../../users/services/users.service';
import { StructureDto } from '../dto/structure.dto';
import { Structure, StructureDocument } from '../schemas/structure.schema';
import { StructuresSearchService } from './structures-search.service';
import { StructuresService } from './structures.service';

function hashPassword() {
  return bcrypt.hashSync(process.env.USER_PWD, process.env.SALT);
}
describe('StructuresService', () => {
  let service: StructuresService;

  const mockStructureModel = {
    create: jest.fn(),
    deleteOne: jest.fn(),
    countDocuments: jest.fn(),
    findOne: jest.fn(),
    exec: jest.fn(),
    find: jest.fn(),
    populate: jest.fn(),
  };

  const structuresSearchServiceMock = {
    search: jest.fn().mockReturnValue([
      {
        _id: '6903ba0e2ab5775cfc01ed4d',
        structureId: '6903ba0e2ab5775cfc01ed4d',
        structureType: null,
        digitalCultureSecurity: ['2', '5', '9', '28', '34', '39', '42', '51', '52', '54', '65', '96', '97', '98'],
        parentingHelp: ['3', '22', '82', '94'],
        socialAndProfessional: ['6', '20', '66', '67', '68', '69', '124', '125', '127'],
        accessRight: ['84', '85', '86', '87', '88', '89', '93', '95'],
        baseSkills: ['260', '1', '11', '38', '48', '74', '77'],
        equipmentsAndServices: ['ordinateurs', 'imprimantes'],
        proceduresAccompaniment: ['cpam', 'impots', 'carsat', 'poleEmploi'],
        publics: ['toutPublic'],
        labelsQualifications: ['passNumerique', 'espacePublicNumeriqueepn'],
        accessModality: ['accesLibre', 'telephoneVisio', 'surRdv'],
        freeWorkShop: false,
        createdAt: '2020-11-16T09:30:00.000Z',
        updatedAt: '2021-04-12T08:48:00.000Z',
        structureName: "L'Atelier Numérique",
        description:
          "L'Atelier Numérique est l'Espace Public Numérique des Centres Sociaux de Rillieux-la-Pape, ayant pour mission la médiation numérique pour toutes et tous.",
        lockdownActivity:
          'accesLibres, permanences numériques téléphoniques, cours et ateliers à distance, formations professionnelles.',
        contactPhone: '',
        contactMail: '',
        website: '',
        facebook: null,
        twitter: null,
        instagram: null,
        pmrAccess: true,
        exceptionalClosures: '',
        jaccompagneLesUsagersDansLeursDemarchesEnLigne: true,
        publicsAccompaniment: [],
        autresAccompagnements: '',
        nbComputers: 16,
        nbPrinters: 1,
        nbTablets: 1,
        nbNumericTerminal: 1,
        hours: {
          monday: {
            open: true,
            time: [
              {
                closing: '12:30',
                opening: '9:00',
              },
              {
                closing: '17:00',
                opening: '13:30',
              },
            ],
          },
          tuesday: {
            open: true,
            time: [
              {
                closing: '12:30',
                opening: '9:00',
              },
              {
                closing: '17:00',
                opening: '13:30',
              },
            ],
          },
          wednesday: {
            open: true,
            time: [
              {
                closing: '12:30',
                opening: '9:00',
              },
              {
                closing: '17:00',
                opening: '13:30',
              },
            ],
          },
          thursday: {
            open: true,
            time: [
              {
                closing: '12:30',
                opening: '9:00',
              },
              {
                closing: '17:00',
                opening: '13:30',
              },
            ],
          },
          friday: {
            open: true,
            time: [
              {
                closing: '12:30',
                opening: '9:00',
              },
            ],
          },
          saturday: {
            open: false,
            time: [],
          },
          sunday: {
            open: false,
            time: [],
          },
        },
        __v: 0,
        address: {
          numero: '30 bis',
          street: 'Avenue Leclerc',
          commune: 'Rillieux-la-Pape',
        },
        coord: [4.9036773, 45.8142196],
        accountVerified: true,
        linkedin: null,
        nbScanners: 1,
        otherDescription: null,
        personalOffers: [],
      },
    ]),
  };

  const mockCategoriesFormationsService = {
    findOneComplete: jest.fn().mockReturnValue({
      _id: '5fbb934180a5c257dc0161f6',
      modules: [
        {
          id: '260',
          display_id: '260',
          display_name: 'Modules APTIC - n°260',
          digest: 'Maitrise de l’environnement d’un ordinateur (clavier, souris)',
          text: 'Maitrise de l’environnement d’un ordinateur (clavier, souris)',
        },
        {
          id: '1',
          display_id: '1',
          display_name: 'Modules APTIC - n°1',
          digest: 'Composantes et facettes de l’identité numérique',
          text: 'Composantes et facettes de l’identité numérique',
        },
        {
          id: '11',
          display_id: '11',
          display_name: 'Modules APTIC - n°11',
          digest: 'Internet : fonctionnement et outils de navigation web',
          text: 'Internet : fonctionnement et outils de navigation web',
        },
        {
          id: '38',
          display_id: '38',
          display_name: 'Modules APTIC - n°38',
          digest: 'Le smartphone : principes de fonctionnement',
          text: 'Le smartphone : principes de fonctionnement',
        },
        {
          id: '48',
          display_id: '48',
          display_name: 'Modules APTIC - n°48',
          digest: 'Internet : envoyer, recevoir, gérer ses emails',
          text: 'Internet : envoyer, recevoir, gérer ses emails',
        },
        {
          id: '74',
          display_id: '74',
          display_name: 'Modules APTIC - n°74',
          digest: 'Smartphones et Tablettes sous Androïd',
          text: 'Smartphones et Tablettes sous Androïd',
        },
        {
          id: '77',
          display_id: '77',
          display_name: 'Modules APTIC - n°77',
          digest: "Smartphone : Les principaux gestes pour l'écran tactile",
          text: "Smartphone : Les principaux gestes pour l'écran tactile",
        },
      ],
      name: 'Les compétences de base',
      id: 'baseSkills',
      __v: 0,
    }),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [HttpModule, MailerModule, SearchModule, ConfigModule],
      providers: [
        StructuresService,
        ConfigurationService,
        ParametersService,
        {
          provide: StructuresSearchService,
          useValue: structuresSearchServiceMock,
        },
        CategoriesFormationsService,
        {
          provide: getModelToken(Structure.name),
          useValue: mockStructureModel,
        },
        {
          provide: CategoriesFormationsService,
          useValue: mockCategoriesFormationsService,
        },
        {
          provide: UsersService,
          useClass: UsersServiceMock,
        },
        {
          provide: getModelToken(Parameters.name),
          useValue: mockParametersModel,
        },
      ],
    }).compile();

    service = module.get<StructuresService>(StructuresService);
    service['structuresSearchService']['index'] = 'structures-unit-test';
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  it('should Initiate structure', () => {
    const res = service.initiateStructureIndex();
    expect(res).toBeTruthy();
  });

  describe('should searchForStructures', () => {
    jest.setTimeout(30000);
    mockStructureModel.find.mockReturnThis();
    mockStructureModel.populate.mockReturnThis();
    mockStructureModel.exec.mockResolvedValue([
      {
        _id: '6903ba0e2ab5775cfc01ed4d',
        structureId: '6903ba0e2ab5775cfc01ed4d',
        structureType: null,
        digitalCultureSecurity: ['2', '5', '9', '28', '34', '39', '42', '51', '52', '54', '65', '96', '97', '98'],
        parentingHelp: ['3', '22', '82', '94'],
        socialAndProfessional: ['6', '20', '66', '67', '68', '69', '124', '125', '127'],
        accessRight: ['84', '85', '86', '87', '88', '89', '93', '95'],
        baseSkills: ['260', '1', '11', '38', '48', '74', '77'],
        equipmentsAndServices: ['ordinateurs', 'imprimantes'],
        proceduresAccompaniment: ['cpam', 'impots', 'carsat', 'poleEmploi'],
        publics: ['toutPublic'],
        labelsQualifications: ['passNumerique', 'espacePublicNumeriqueepn'],
        accessModality: ['accesLibre', 'telephoneVisio', 'surRdv'],
        freeWorkShop: false,
        createdAt: '2020-11-16T09:30:00.000Z',
        updatedAt: '2021-04-12T08:48:00.000Z',
        structureName: "L'Atelier Numérique",
        description:
          "L'Atelier Numérique est l'Espace Public Numérique des Centres Sociaux de Rillieux-la-Pape, ayant pour mission la médiation numérique pour toutes et tous.",
        lockdownActivity:
          'accesLibres, permanences numériques téléphoniques, cours et ateliers à distance, formations professionnelles.',
        contactPhone: '',
        contactMail: '',
        website: '',
        facebook: null,
        twitter: null,
        instagram: null,
        pmrAccess: true,
        exceptionalClosures: '',
        jaccompagneLesUsagersDansLeursDemarchesEnLigne: true,
        publicsAccompaniment: [],
        autresAccompagnements: '',
        nbComputers: 16,
        nbPrinters: 1,
        nbTablets: 1,
        nbNumericTerminal: 1,
        hours: {
          monday: {
            open: true,
            time: [
              {
                closing: '12:30',
                opening: '9:00',
              },
              {
                closing: '17:00',
                opening: '13:30',
              },
            ],
          },
        },
        __v: 0,
        address: {
          numero: '30 bis',
          street: 'Avenue Leclerc',
          commune: 'Rillieux-la-Pape',
        },
        coord: [4.9036773, 45.8142196],
        accountVerified: true,
        linkedin: null,
        nbScanners: 1,
        otherDescription: null,
        personalOffers: [],
      },
    ]);

    it('should find 1 structure', async () => {
      const res = await service.searchForStructures('a', [{ nbPrinters: '1' }]);
      expect(res.length).toBe(1);
    });
    it('should find 1 structure', async () => {
      const res = await service.searchForStructures('a', [{ nbPrinters: '1' }, { '': 'baseSkills' }]);
      expect(res.length).toBe(1);
    });
    it('should find 1 structure', async () => {
      const res = await service.searchForStructures('a', [{ '': 'baseSkills' }]);
      expect(res.length).toBe(1);
    });
    it('should find 1 structure', async () => {
      const res = await service.searchForStructures('a');
      expect(res.length).toBe(1);
    });
  });

  it('should create structure', () => {
    const structure = new StructureDto();
    let res = service.create(null, structure);
    expect(res).toBeTruthy();
    res = service.create('tsfsf6296', structure);
    expect(res).toBeTruthy();
  });

  it('should search structure', () => {
    const filters = [{ nbPrinters: '1' }];
    let res = service.search('', filters);
    expect(res).toBeTruthy();
    res = service.search(null, filters);
    expect(res).toBeTruthy();
    res = service.search(null);
    expect(res).toBeTruthy();
  });

  it('should find all structures', () => {
    const res = service.findAll();
    expect(res).toBeTruthy();
  });

  it('should find all unclaimed structures', () => {
    const res = service.findAllUnclaimed();
    expect(res).toBeTruthy();
  });

  it('should find all formated structures', () => {
    const res = service.findAllFormated(null, null, null);
    expect(res).toBeTruthy();
  });

  it('should populate ES', () => {
    const res = service.populateES();
    expect(res).toBeTruthy();
  });

  describe('getAllDataConsentPendingStructures', () => {
    it('should get one empty consent', async () => {
      const user = {
        _id: '123',
        validationToken:
          'cf1c74c22cedb6b575945098db42d2f493fb759c9142c6aff7980f252886f36ee086574ee99a06bc99119079257116c959c8ec870949cebdef2b293666dbca42',
        emailVerified: false,
        email: 'jacques.dupont@mii.com',
        password: hashPassword(),
        role: 0,
        newEmail: '',
        changeEmailToken: '',
        resetPasswordToken: null,
        structuresLink: [
          Types.ObjectId('620e510b7d8f26712c386be7'),
          Types.ObjectId('620e51247d8f26712c386be9'),
          Types.ObjectId('620e5236f25755550cb86dfd'),
        ],
        pendingStructuresLink: [],
        structureOutdatedMailSent: [],
        personalOffers: [],
        name: 'Jacques',
        surname: 'Dupont',
        phone: '06 06 06 06 06',
        createdAt: new Date('2022-05-25T09:48:28.824Z'),
      } as IUser;
      jest
        .spyOn(service, 'findOne')
        .mockResolvedValueOnce({
          _id: Types.ObjectId('61e9260b2ac971550065e261'),
          coord: [4.844309, 45.865288],
          createdAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          updatedAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          __v: 0,
        } as StructureDocument)
        .mockResolvedValueOnce({
          _id: Types.ObjectId('61e9260b2ac971550065e261'),
          coord: [4.844309, 45.865288],
          createdAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          updatedAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          dataShareConsentDate: null,
        } as StructureDocument)
        .mockResolvedValueOnce({
          _id: Types.ObjectId('61e9260b2ac971550065e261'),
          coord: [4.844309, 45.865288],
          createdAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          updatedAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          dataShareConsentDate: new Date(),
        } as StructureDocument);
      const res = await service.getAllDataConsentPendingStructures(user);
      expect(res.length).toBe(1);
    });
    it('should get no consent', async () => {
      const user = {
        _id: '123',
        validationToken:
          'cf1c74c22cedb6b575945098db42d2f493fb759c9142c6aff7980f252886f36ee086574ee99a06bc99119079257116c959c8ec870949cebdef2b293666dbca42',
        emailVerified: false,
        email: 'jacques.dupont@mii.com',
        password: hashPassword(),
        role: 0,
        newEmail: '',
        changeEmailToken: '',
        resetPasswordToken: null,
        structuresLink: [
          Types.ObjectId('620e510b7d8f26712c386be7'),
          Types.ObjectId('620e51247d8f26712c386be9'),
          Types.ObjectId('620e5236f25755550cb86dfd'),
        ],
        pendingStructuresLink: [],
        structureOutdatedMailSent: [],
        personalOffers: [],
        name: 'Jacques',
        surname: 'Dupont',
        phone: '06 06 06 06 06',
        createdAt: new Date('2022-05-25T09:48:28.824Z'),
      } as IUser;
      jest
        .spyOn(service, 'findOne')
        .mockResolvedValueOnce({
          _id: Types.ObjectId('61e9260b2ac971550065e261'),
          coord: [4.844309, 45.865288],
          createdAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          updatedAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          dataShareConsentDate: null,
          __v: 0,
        } as StructureDocument)
        .mockResolvedValueOnce({
          _id: Types.ObjectId('61e9260b2ac971550065e261'),
          coord: [4.844309, 45.865288],
          createdAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          updatedAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          dataShareConsentDate: null,
        } as StructureDocument)
        .mockResolvedValueOnce({
          _id: Types.ObjectId('61e9260b2ac971550065e261'),
          coord: [4.844309, 45.865288],
          createdAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          updatedAt: new Date('Thu Jan 20 2022 10:06:19 GMT+0100 (heure normale d’Europe centrale)'),
          dataShareConsentDate: new Date(),
        } as StructureDocument);
      const res = await service.getAllDataConsentPendingStructures(user);
      expect(res.length).toBe(0);
    });
  });

  it('should report structure Error', () => {
    let res = service.reportStructureError('6093ba0e2ab5775cfc01ed3e', '');
    expect(res).toBeTruthy();
    res = service.reportStructureError(null, '');
    expect(res).toBeTruthy();
  });

  describe('addPersonalOffer', () => {
    const personalOfferDocumentMock: PersonalOfferDocument = personalOffersDataMock[0] as PersonalOfferDocument;
    it('should add personal offer to the structure', async () => {
      jest.spyOn(service, 'findOne').mockResolvedValue(structuresDocumentDataMock[0]);
      const expectedResult = { ...structuresDocumentDataMock[0], personalOffers: [personalOfferDocumentMock] };
      expect(await service.addPersonalOffer('6093ba0e2ab5775cfc01ed3e', personalOfferDocumentMock)).toEqual(
        expectedResult
      );
    });
    it('should return exception if structure is not found for given id', async () => {
      jest.spyOn(service, 'findOne').mockResolvedValue(null);
      try {
        await service.addPersonalOffer('abcd', personalOfferDocumentMock);
        // Fail test if above expression doesn't throw anything.
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Structure not found for the personal offer attachment');
        expect(e.status).toBe(400);
      }
    });
    it('should return exception if personal offer already exists in the structure', async () => {
      jest.spyOn(service, 'findOne').mockResolvedValue(structuresDocumentDataMock[1]);
      try {
        await service.addPersonalOffer('6093ba0e2ab5775cfc01ed3e', personalOfferDocumentMock);
        // Fail test if above expression doesn't throw anything.
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Personal offer already exist in the structure');
        expect(e.status).toBe(400);
      }
    });
  });
});
