import { Injectable } from '@nestjs/common';
import { ElasticsearchService } from '@nestjs/elasticsearch';
import { StructureDto } from '../dto/structure.dto';
import { StructureDocument } from '../schemas/structure.schema';
import { StructureSearchBody } from '../interfaces/structure-search-body.interface';
import { StructureSearchResult } from '../interfaces/structure-search-response.interface';

@Injectable()
export class StructuresSearchService {
  private index = 'structures';

  constructor(private readonly elasticsearchService: ElasticsearchService) {}

  public async indexStructure(structure: StructureDocument): Promise<StructureDocument> {
    this.elasticsearchService.index<StructureSearchResult, StructureSearchBody>({
      index: this.index,
      id: structure._id,
      body: {
        structureName: structure.structureName,
        structureType: structure.structureType,
        structureId: structure._id,
        address: structure.address,
        description: structure.description,
      },
    });
    return structure;
  }

  public async createStructureIndex(): Promise<any> {
    // use custom analyzer with minimal_french stemmer to avoid default light_french stemmer problems (oullins -> oulin, "oull" not found)
    // don't use stopwords (le, la, de, son, etc.) for structureName (to find "maison de la", "le son du clic", etc.)
    return this.elasticsearchService.indices.create({
      index: this.index,
      body: {
        mappings: {
          dynamic_templates: [
            {
              strings: {
                match_mapping_type: 'string',
                mapping: {
                  type: 'text',
                  analyzer: 'homemade_french',
                  search_analyzer: 'homemade_french',
                },
              },
            },
          ],
          properties: {
            structureName: {
              type: 'text',
              analyzer: 'homemade_french_stopless',
              search_analyzer: 'homemade_french_stopless',
            },
          },
        },
        settings: {
          analysis: {
            filter: {
              french_elision: {
                type: 'elision',
                articles_case: true,
                articles: ['l', 'm', 't', 'qu', 'n', 's', 'j', 'd', 'c', 'jusqu', 'quoiqu', 'lorsqu', 'puisqu'],
              },
              french_stop: {
                type: 'stop',
                stopwords: '_french_',
              },
              french_stemmer: {
                type: 'stemmer',
                language: 'minimal_french',
              },
            },
            analyzer: {
              homemade_french: {
                tokenizer: 'standard',
                filter: ['lowercase', 'asciifolding', 'french_elision', 'french_stop', 'french_stemmer'],
              },
              homemade_french_stopless: {
                tokenizer: 'standard',
                filter: ['lowercase', 'asciifolding', 'french_elision', 'french_stemmer'],
              },
            },
          },
        },
      },
    });
  }

  public async dropIndex(): Promise<any> {
    if (
      (
        await this.elasticsearchService.indices.exists({
          index: this.index,
        })
      ).body
    ) {
      return this.elasticsearchService.indices.delete({
        index: this.index,
      });
    }
  }

  public async deleteIndexStructure(structure: StructureDocument): Promise<StructureDocument> {
    this.elasticsearchService.delete<StructureSearchResult, StructureSearchBody>({
      index: this.index,
      id: structure._id,
    });
    return structure;
  }

  public async refreshIndexStructure(): Promise<any> {
    return this.elasticsearchService.indices.refresh({
      index: this.index,
    });
  }

  public async search(searchString: string, fields?: string[]): Promise<StructureSearchBody[]> {
    searchString = searchString ? searchString + '*' : '*';
    const { body } = await this.elasticsearchService.search<StructureSearchResult>({
      index: this.index,
      body: {
        from: 0,
        size: 200,
        query: {
          query_string: {
            analyze_wildcard: 'true',
            query: searchString,
            fields: fields ? fields : ['structureName^5', 'structureType^5', 'address.commune^10', 'description'],
            fuzziness: 'AUTO',
          },
        },
      },
    });
    const sortedHits = body.hits.hits.filter(function (elem) {
      return elem._score >= body.hits.max_score / 1.5;
    });
    return sortedHits.map((item) => item._source);
  }

  public async update(structure: StructureDto, id: string): Promise<any> {
    return this.elasticsearchService.update({
      index: this.index,
      id: id,
      body: {
        doc: {
          structureName: structure.structureName,
          structureType: structure.structureType,
          structureId: id,
          address: structure.address,
          description: structure.description,
        },
      },
    });
  }
}
