import { HttpModule, HttpStatus } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { Types } from 'mongoose';
import { UsersServiceMock } from '../../../test/mock/services/user.mock.service';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { Employer } from '../schemas/employer.schema';
import { EmployerService } from '../services/employer.service';
import { UsersService } from '../services/users.service';
import { EmployerController } from './employer.controller';

describe('EmployerController', () => {
  let controller: EmployerController;
  let userService: UsersService;

  const employerServiceMock = {
    findAll: jest.fn(),
    searchByName: jest.fn(),
    findByName: jest.fn(),
    create: jest.fn(),
    deleteByName: jest.fn(),
    initEmployerIndex: jest.fn(),
    findOne: jest.fn(),
    deleteOneId: jest.fn(),
    findAllValidated: jest.fn(),
    findAllUnvalidated: jest.fn(),
    validate: jest.fn(),
    update: jest.fn(),
    mergeEmployer: jest.fn(),
    deleteInvalidEmployer: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [ConfigurationModule, HttpModule],
      providers: [
        {
          provide: UsersService,
          useClass: UsersServiceMock,
        },
        {
          provide: EmployerService,
          useValue: employerServiceMock,
        },
        {
          provide: getModelToken('Employer'),
          useValue: Employer,
        },
      ],
      controllers: [EmployerController],
    }).compile();

    controller = module.get<EmployerController>(EmployerController);
    userService = module.get<UsersService>(UsersService);
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  describe('findAll', () => {
    it('should return two elements ', async () => {
      employerServiceMock.findAll.mockResolvedValueOnce([
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5a7'),
          name: 'CAF',
          validated: true,
        },
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5a7'),
          name: 'CARSAT',
          validated: true,
        },
      ]);
      const findAllReply = await controller.findAll();
      expect(findAllReply.length).toBe(2);
    });
    it('should return searched elements with query `CA`', async () => {
      employerServiceMock.searchByName.mockResolvedValueOnce([
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5a7'),
          name: 'CAF',
          validated: true,
        },
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5a7'),
          name: 'CARSAT',
          validated: true,
        },
      ]);
      const findAllReply = await controller.findAll({ search: 'CA' });
      expect(findAllReply.length).toBe(2);
    });
  });

  // describe('deleteEmployer', () => {
  //   it('should delete employer `Metro`', async () => {
  //     const employer: Employer = {
  //       name: 'Metro',
  //       validated: true,
  //     };
  //     const employerToRemove: CreateEmployerDto = {
  //       name: 'Metro',
  //     };
  //     employerServiceMock.findByName.mockResolvedValueOnce(employer);
  //     employerServiceMock.deleteByName.mockResolvedValueOnce({
  //       ok: 1,
  //       n: 1,
  //       deletedCount: 1,
  //     });
  //     const deleteEmployer = await controller.deleteEmployer(employerToRemove);
  //     expect(deleteEmployer.name).toBe('Metro');
  //     expect(deleteEmployer.validated).toBe(true);
  //   });

  //   it('should throw error on unexisting employer `Metro`', async () => {
  //     const employerToRemove: CreateEmployerDto = {
  //       name: 'Metro',
  //     };
  //     employerServiceMock.deleteByName.mockResolvedValueOnce(null);
  //     try {
  //       await controller.deleteEmployer(employerToRemove);
  //       expect(true).toBe(false);
  //     } catch (e) {
  //       expect(e.message).toBe('Employer does not exist');
  //       expect(e.status).toBe(404);
  //     }
  //   });
  // });
  describe('resetES', () => {
    it('should reset search index', async () => {
      employerServiceMock.initEmployerIndex.mockResolvedValueOnce([
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5a7'),
          name: 'CAF',
          validated: true,
        },
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5a7'),
          name: 'CARSAT',
          validated: true,
        },
      ]);
      const index = await controller.resetES();
      expect(index.length).toBe(2);
    });
  });

  describe('Create Employer', () => {
    it('should create a employer', async () => {
      employerServiceMock.findByName.mockResolvedValueOnce(null);
      employerServiceMock.create.mockResolvedValueOnce({
        _id: Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Sopra',
        validated: true,
      });
      const req = { user: { _id: '6036721022462b001334c4bb', role: 0 } };
      const reply = await controller.createEmployer({ name: 'Sopra' }, req);
      expect(reply).toBeTruthy();
    });
    it('should not create if employer already exists', async () => {
      employerServiceMock.findByName.mockResolvedValueOnce({
        _id: Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Sopra',
        validated: true,
      });
      const req = { user: { _id: '6036721022462b001334c4bb' }, role: 0 };
      try {
        await controller.createEmployer({ name: 'Sopra' }, req);
        expect;
      } catch (e) {
        expect(e.message).toBe('Employer already exist');
        expect(e.status).toBe(HttpStatus.UNPROCESSABLE_ENTITY);
      }
    });
    it('should call create with send notification to true if admin', async () => {
      employerServiceMock.findByName.mockResolvedValueOnce(null);
      const employer = {
        name: 'Sopra',
      };
      employerServiceMock.create.mockResolvedValueOnce({ ...employer, validated: true });
      const req = { user: { _id: '6036721022462b001334c4bb', role: 1 } };

      const reply = await controller.createEmployer(employer, req);
      expect(reply).toBeTruthy();
      expect(employerServiceMock.create).toHaveBeenCalledWith(employer, true, false);
    });
  });
  describe('Validate Employer', () => {
    it('should validate an employer', async () => {
      employerServiceMock.validate.mockResolvedValueOnce({
        _id: Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Sopra',
        validated: true,
      });
      expect(await controller.validateEmployer({ id: '6231aefe76598527c8d0b5bc' })).toBeTruthy();
    });
  });
  describe('Get Employers', () => {
    it('should call all validated employers and populate them with users attached to it', async () => {
      const spyer = jest.spyOn(userService, 'populateEmployerswithUsers');
      employerServiceMock.findAllValidated.mockResolvedValueOnce([
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Metro',
          validated: true,
        },
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Sopra',
          validated: true,
        },
      ]);
      await controller.findValidatedEmployers();
      expect(employerServiceMock.findAllValidated.mock.calls.length).toBe(1);
      expect(spyer.mock.calls.length).toBe(1);
    });

    it('should call all unvalidated employers and populate them with users attached to it', async () => {
      const spyer = jest.spyOn(userService, 'populateEmployerswithUsers');
      employerServiceMock.findAllUnvalidated.mockResolvedValueOnce([
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Metro',
          validated: false,
        },
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'Sopra',
          validated: false,
        },
      ]);
      await controller.findUnvalidatedEmployers();
      expect(employerServiceMock.findAllUnvalidated.mock.calls.length).toBe(1);
      expect(spyer.mock.calls.length).toBe(1);
    });
  });
  describe('Edit Employer', () => {
    it('should update employer', async () => {
      employerServiceMock.update.mockResolvedValueOnce({
        _id: Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'SopraMod',
        validated: true,
      });
      expect(await controller.updateEmployer('6231aefe76598527c8d0b5bc', { name: 'SopraMod' })).toBeTruthy();
    });

    it('should delete an unvalidated employer and replace all its occurence with a chosen validated employer', async () => {
      employerServiceMock.mergeEmployer.mockResolvedValueOnce({
        _id: Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Sopra',
        validated: true,
      });
      employerServiceMock.deleteInvalidEmployer.mockResolvedValueOnce({
        n: 1,
        ok: 1,
        deletedCount: 1,
      });

      const reply = await controller.mergeEmployer({
        sourceEmployerId: '6231aefe76598527c8d0b5ba',
        targetEmployerId: '6231aefe76598527c8d0b5bc',
      });
      expect(reply).toBeTruthy();
    });
  });
  describe('Delete Employer', () => {
    it('should delete employer', async () => {
      employerServiceMock.deleteOneId.mockResolvedValueOnce({
        n: 1,
        ok: 1,
        deletedCount: 1,
      });

      employerServiceMock.findOne.mockResolvedValueOnce({
        _id: Types.ObjectId('6231aefe76598527c8d0b5ba'),
        name: 'Sopra',
        validated: true,
      });
      const reply = await controller.deleteEmployer({ id: '6231aefe76598527c8d0b5ba' });
      expect(reply).toBeTruthy();
    });

    it('should not delete employer if a user is linked', async () => {
      employerServiceMock.findOne.mockResolvedValueOnce({
        _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Sopra',
        validated: true,
      });
      try {
        await controller.deleteEmployer({ id: '6231aefe76598527c8d0b5bc' });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toEqual('Cannot delete employer. It has user(s) attached to it.');
        expect(e.status).toEqual(HttpStatus.FORBIDDEN);
      }
    });
  });
});
