import {
  Body,
  Controller,
  Delete,
  Get,
  HttpException,
  HttpStatus,
  Logger,
  Param,
  Post,
  Put,
  Request,
  UseGuards,
} from '@nestjs/common';
import { ApiBearerAuth, ApiOperation, ApiParam, ApiTags } from '@nestjs/swagger';
import { MergeJobDto } from '../../admin/dto/merge-job.dto';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';
import { hasAdminRole } from '../../shared/utils';
import { Roles } from '../decorators/roles.decorator';
import { CreateJobDto } from '../dto/create-job.dto';
import { RolesGuard } from '../guards/roles.guard';
import { IJob } from '../interfaces/job.interface';
import { Job } from '../schemas/job.schema';
import { JobsService } from '../services/jobs.service';
import { UsersService } from '../services/users.service';

@ApiTags('jobs')
@Controller('jobs')
export class JobsController {
  private readonly logger = new Logger(JobsController.name);

  constructor(private jobsService: JobsService, private usersService: UsersService) {}

  /**
   * Return every jobs
   */
  @Get()
  public async findAll(): Promise<any> {
    this.logger.debug('findAll');
    return this.jobsService.findAll();
  }

  /**
   * Create a new job
   * @param job {CreateJobDto}
   * @returns {Job}
   */
  @Post()
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth('JWT')
  @ApiParam({ name: 'job', type: CreateJobDto, required: true })
  public async createJob(@Request() req, @Body() job: CreateJobDto): Promise<Job> {
    this.logger.debug(`createJob: ${job.name}`);
    const existingJob = await this.jobsService.findByName(job.name);
    if (existingJob) {
      this.logger.warn(`Job already exist: ${job.name}`);
      throw new HttpException('Job already exist', HttpStatus.UNPROCESSABLE_ENTITY);
    }
    // if user is admin, do not send notification
    if (hasAdminRole(req.user)) {
      return this.jobsService.create(job, true, job.hasPersonalOffer, false);
    }
    return this.jobsService.create(job);
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Get('validated')
  @ApiOperation({ description: 'Get validated jobs populated with users attached to it' })
  public async findValidatedJobs() {
    return this.jobsService.findAll().then(async (jobs: IJob[]) => {
      return this.usersService.populateJobswithUsers(jobs);
    });
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Get('unvalidated')
  @ApiOperation({ description: 'Get unvalidated jobs populated with users attached to it' })
  public async findUnvalidatedJobs() {
    return this.jobsService.findAllUnvalidated().then(async (jobs: IJob[]) => {
      return this.usersService.populateJobswithUsers(jobs);
    });
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Post('validate/:id')
  @ApiParam({ name: 'id', type: String, required: true })
  @ApiOperation({ description: 'Validate job' })
  public async validateJob(@Param() params) {
    return this.jobsService.validate(params.id);
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Put('merge')
  public async mergeJob(@Body() mergeJob: MergeJobDto) {
    this.logger.debug('mergeJob');
    return this.jobsService.mergeJob(mergeJob);
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @ApiBearerAuth('JWT')
  @Roles('admin')
  @Put(':id')
  public async updateJob(@Param('id') id: string, @Body() body: CreateJobDto) {
    this.logger.debug('updateJob');
    return this.jobsService.update(id, body);
  }

  @UseGuards(JwtAuthGuard, RolesGuard)
  @Roles('admin')
  @Delete(':id')
  @ApiBearerAuth('JWT')
  @ApiParam({ name: 'id', type: String, required: true })
  public async deleteJob(@Param() params) {
    // look for job
    const researchedJob = await this.jobsService.findOne(params.id);
    // look for any relations within user collection, reject action if found
    if (researchedJob !== null) {
      const isJobLinked = await this.usersService.isJobLinkedtoUser(researchedJob._id);
      if (!isJobLinked) {
        return this.jobsService.deleteOneId(params.id);
      } else {
        throw new HttpException('Cannot delete job. It has user(s) attached to it.', HttpStatus.FORBIDDEN);
      }
    } else {
      throw new HttpException('Job does not exists', HttpStatus.NOT_FOUND);
    }
  }
}
