import { HttpModule } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { Types } from 'mongoose';
import { CategoriesFormationsServiceMock } from '../../../test/mock/services/categoriesFormations.mock.service';
import { CategoriesFormationsService } from '../../categories/services/categories-formations.service';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { MailerService } from '../../mailer/mailer.service';
import { ParametersService } from '../../parameters/parameters.service';
import { Parameters } from '../../parameters/schemas/parameters.schema';
import { SearchModule } from '../../search/search.module';
import { Structure } from '../../structures/schemas/structure.schema';
import { StructuresSearchService } from '../../structures/services/structures-search.service';
import { StructuresService } from '../../structures/services/structures.service';
import { TempUser } from '../../temp-user/temp-user.schema';
import { TempUserService } from '../../temp-user/temp-user.service';
import { ProfileDto } from '../dto/profile.dto';
import { UpdateDetailsDto } from '../dto/update-details.dto';
import { User } from '../schemas/user.schema';
import { EmployerService } from '../services/employer.service';
import { JobsService } from '../services/jobs.service';
import { UsersService } from '../services/users.service';
import { UsersController } from './users.controller';

describe('UsersController', () => {
  let controller: UsersController;

  const employerServiceMock = {
    findByName: jest.fn(),
  };

  const jobServiceMock = {
    findByName: jest.fn(),
  };

  const userServiceMock = {
    updateUserProfile: jest.fn(),
    findOne: jest.fn(),
    updateUserDetails: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [ConfigurationModule, HttpModule, SearchModule],
      providers: [
        UsersService,
        StructuresService,
        StructuresSearchService,
        MailerService,
        TempUserService,
        ParametersService,
        {
          provide: UsersService,
          useValue: userServiceMock,
        },
        {
          provide: EmployerService,
          useValue: employerServiceMock,
        },
        {
          provide: JobsService,
          useValue: jobServiceMock,
        },
        {
          provide: CategoriesFormationsService,
          useValue: CategoriesFormationsServiceMock,
        },
        {
          provide: getModelToken('TempUser'),
          useValue: TempUser,
        },
        {
          provide: getModelToken('Structure'),
          useValue: Structure,
        },
        {
          provide: getModelToken('User'),
          useValue: User,
        },
        {
          provide: getModelToken('Parameters'),
          useValue: Parameters,
        },
      ],
      controllers: [UsersController],
    }).compile();

    controller = module.get<UsersController>(UsersController);
  });

  it('should be defined', () => {
    expect(controller).toBeDefined();
  });

  describe('setProfile', () => {
    it('should return employer does not exist', async () => {
      const profile: ProfileDto = {
        employerName: 'Metro',
        jobName: 'Dev',
      };
      employerServiceMock.findByName.mockResolvedValueOnce(null);
      try {
        await controller.setProfile({ user: { _id: '36', email: 'a@a.com' } }, profile);
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Employer does not exist');
        expect(e.status).toBe(400);
      }
    });
    it('should return job does not exist', async () => {
      const profile: ProfileDto = {
        employerName: 'Metro',
        jobName: 'Dev',
      };
      employerServiceMock.findByName.mockResolvedValueOnce({
        _id: Types.ObjectId('6231aefe76598527c8d0b5a7'),
        name: 'CAF',
        validated: true,
      });
      jobServiceMock.findByName.mockResolvedValue(null);
      try {
        await controller.setProfile({ user: { _id: '36', email: 'a@a.com' } }, profile);
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Job does not exist');
        expect(e.status).toBe(400);
      }
    });
    it('should update profile', async () => {
      const profile: ProfileDto = {
        employerName: 'Metro',
        jobName: 'Dev',
      };
      employerServiceMock.findByName.mockResolvedValueOnce({
        _id: Types.ObjectId('6231aefe76598527c8d0b5a7'),
        name: 'CAF',
        validated: true,
      });
      jobServiceMock.findByName.mockResolvedValue({
        _id: Types.ObjectId('6231aefe76598527c8d0b5be'),
        name: 'Dev',
        validated: false,
        hasPersonalOffer: false,
      });
      userServiceMock.findOne.mockResolvedValueOnce({
        _id: Types.ObjectId('620e4ffa72389045b02ac854'),
        structuresLink: [Types.ObjectId('620e5236f25755550cb86dfd')],
        structureOutdatedMailSent: [],
        pendingStructuresLink: [Types.ObjectId('61e926192ac971550065e275')],
        phone: '06 06 06 06 06',
        newEmail: null,
        changeEmailToken: null,
        role: 1,
        resetPasswordToken: null,
        surname: 'ADMIN',
        name: 'Admin',
        validationToken: null,
        emailVerified: true,
        email: 'admin@admin.com',
        password: '$2a$12$vLQjJ9zAWyUwiXLeQDa6w.XzrlgPBhw.2GWrjog/yuEjIaZnQwmZu',
        __v: 11,
        employer: Types.ObjectId('6231aefe76598527c8d0b5a7'),
        job: Types.ObjectId('6231aefe76598527c8d0b5be'),
      });
      const reply = await controller.setProfile({ user: { _id: '36', email: 'a@a.com' } }, profile);
      console.log(reply);
      expect(reply).toBeTruthy();
    });
  });

  describe('update details', () => {
    it('should update details', async () => {
      const newDetails: UpdateDetailsDto = {
        name: 'Jean',
        surname: 'Jacque',
        phone: '0605040302',
      };
      userServiceMock.updateUserDetails.mockResolvedValueOnce({
        _id: Types.ObjectId('620e4ffa72389045b02ac854'),
        structuresLink: [Types.ObjectId('620e5236f25755550cb86dfd')],
        structureOutdatedMailSent: [],
        pendingStructuresLink: [Types.ObjectId('61e926192ac971550065e275')],
        phone: '06 06 06 06 06',
        newEmail: null,
        changeEmailToken: null,
        role: 1,
        resetPasswordToken: null,
        surname: 'ADMIN',
        name: 'Admin',
        validationToken: null,
        emailVerified: true,
        email: 'admin@admin.com',
        password: '$2a$12$vLQjJ9zAWyUwiXLeQDa6w.XzrlgPBhw.2GWrjog/yuEjIaZnQwmZu',
        __v: 11,
        employer: Types.ObjectId('6231aefe76598527c8d0b5a7'),
        job: Types.ObjectId('6231aefe76598527c8d0b5be'),
      });
      const reply = await controller.updateDetails({ user: { _id: '36', email: 'a@a.com' } }, newDetails);
      console.log(reply);
      expect(reply).toBeTruthy();
    });
  });
});
