import {
  Body,
  Controller,
  Delete,
  Get,
  HttpException,
  HttpStatus,
  Logger,
  Param,
  Post,
  Query,
  Req,
  Request,
  UseGuards,
} from '@nestjs/common';
import { ApiBearerAuth, ApiBody, ApiOperation, ApiParam, ApiResponse, ApiTags } from '@nestjs/swagger';
import { JwtAuthGuard } from '../../auth/guards/jwt-auth.guard';
import { PasswordChangeDto } from '../dto/change-password.dto';
import { EmailChangeDto } from '../dto/change-email.dto';
import { CreateUserDto } from '../dto/create-user.dto';
import { PasswordResetApplyDto } from '../dto/reset-password-apply.dto';
import { PasswordResetDto } from '../dto/reset-password.dto';
import { UsersService } from '../services/users.service';
import { StructuresService } from '../../structures/services/structures.service';
import { TempUserService } from '../../temp-user/temp-user.service';
import { ConfigurationService } from '../../configuration/configuration.service';
import { Structure } from '../../structures/schemas/structure.schema';
import { ProfileDto } from '../dto/profile.dto';
import { EmployerService } from '../services/employer.service';
import { JobsService } from '../services/jobs.service';
import { IUser } from '../interfaces/user.interface';
import { UpdateDetailsDto } from '../dto/update-details.dto';
import { User } from '../schemas/user.schema';
import { DescriptionDto } from '../dto/description.dto';
@ApiTags('users')
@Controller('users')
export class UsersController {
  private readonly logger = new Logger(UsersController.name);
  constructor(
    private usersService: UsersService,
    private structureService: StructuresService,
    private tempUserService: TempUserService,
    private employerService: EmployerService,
    private jobsService: JobsService,
    private configurationService: ConfigurationService
  ) {}

  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth('JWT')
  @ApiOperation({ description: 'Get user profile' })
  @ApiResponse({ status: HttpStatus.OK, description: 'Return user profil' })
  @ApiResponse({ status: HttpStatus.UNAUTHORIZED, description: 'User does not have sufficient rights' })
  @Get('profile')
  public getProfile(@Request() req) {
    return req.user;
  }

  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth('JWT')
  @ApiOperation({ description: 'Set user profile with employer and job' })
  @ApiResponse({ status: HttpStatus.OK, description: 'Return user profil' })
  @ApiResponse({ status: HttpStatus.BAD_REQUEST, description: 'Job does not exist' })
  @ApiResponse({ status: HttpStatus.BAD_REQUEST, description: 'Employer does not exist' })
  @Post('profile')
  public async setProfile(@Request() req, @Body() profile: ProfileDto): Promise<IUser> {
    this.logger.debug(`setProfile`);
    const employerDocument = await this.employerService.findByName(profile.employerName);
    if (!employerDocument) {
      this.logger.warn(`Employer does not exist: ${profile.employerName}`);
      throw new HttpException('Employer does not exist', HttpStatus.BAD_REQUEST);
    }
    const jobDocument = await this.jobsService.findByName(profile.jobName);
    if (!jobDocument) {
      this.logger.warn(`Job does not exist: ${profile.jobName}`);
      throw new HttpException('Job does not exist', HttpStatus.BAD_REQUEST);
    }
    await this.usersService.updateUserProfile(req.user._id, employerDocument, jobDocument);
    return this.usersService.findOne(req.user.email);
  }

  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth('JWT')
  @ApiOperation({ description: 'Updates name, surname and phone number of a user' })
  @ApiBody({ type: UpdateDetailsDto })
  @ApiResponse({ status: HttpStatus.CREATED, description: 'Return user profil' })
  @ApiResponse({ status: HttpStatus.NOT_FOUND, description: 'User not found' })
  @ApiResponse({ status: HttpStatus.BAD_REQUEST, description: 'Missing parametter' })
  @Post('details')
  public async updateDetails(@Request() req, @Body() details: UpdateDetailsDto): Promise<User | HttpException> {
    this.logger.debug('updateProfile details');
    return this.usersService.updateUserDetails(req.user._id, details);
  }

  @Post()
  @ApiResponse({ status: 201, description: 'User created' })
  public async create(@Body() createUserDto: CreateUserDto) {
    // remove structureId for creation and add structure after
    let structureId = null;
    if (createUserDto.pendingStructuresLink.length > 0) {
      structureId = createUserDto.pendingStructuresLink[0];
      delete createUserDto.pendingStructuresLink;
    }
    const user = await this.usersService.create(createUserDto);
    if (structureId) {
      const structure = await this.structureService.findOne(structureId);
      this.usersService.updateStructureLinkedClaim(createUserDto.email, structureId, structure);
      this.structureService.sendAdminStructureNotification(
        null,
        this.configurationService.config.templates.adminStructureClaim.ejs,
        this.configurationService.config.templates.adminStructureClaim.json,
        user
      );
    }
    // Remove temp user if exist
    const tempUser = await this.tempUserService.findOne(createUserDto.email);
    if (tempUser) {
      this.tempUserService.delete(createUserDto.email);
    }
    return user;
  }

  @Post('verify/:id')
  @ApiParam({ name: 'id', type: String, required: true })
  @ApiResponse({ status: 201, description: 'User verified' })
  @ApiResponse({ status: 401, description: "This token does'nt exist or is not associate to this user." })
  public async validateUser(@Param() params, @Query('token') token: string) {
    return this.usersService.validateUser(params.id, token);
  }

  @UseGuards(JwtAuthGuard)
  @Post('change-password')
  @ApiResponse({ status: 201, description: 'Password changed' })
  @ApiResponse({ status: 401, description: 'Invalid password' })
  @ApiResponse({ status: 422, description: 'Weak password' })
  public async changePassword(@Request() req, @Body() passwordChangeDto: PasswordChangeDto) {
    return this.usersService.changeUserPassword(
      req.user._id,
      passwordChangeDto.oldPassword,
      passwordChangeDto.newPassword
    );
  }

  @UseGuards(JwtAuthGuard)
  @Post('change-email')
  @ApiResponse({ status: 201, description: 'Email confirmation send' })
  @ApiResponse({ status: 401, description: 'Invalid Email' })
  public async changeEmail(@Request() _req, @Body() emailChangeDto: EmailChangeDto) {
    return this.usersService.changeUserEmail(emailChangeDto);
  }

  @UseGuards(JwtAuthGuard)
  @Post('verify-change-email')
  @ApiResponse({ status: 201, description: 'Email changed' })
  @ApiResponse({ status: 401, description: 'Invalid Token' })
  public async verifyAndUpdateEmail(@Request() _req, @Query('token') token: string) {
    return this.usersService.verifyAndUpdateUserEmail(token);
  }

  @Post('reset-password')
  @ApiResponse({ status: 200, description: 'Email sent if account exist' })
  public async resetPassword(@Body() passwordReset: PasswordResetDto) {
    return this.usersService.sendResetPasswordEmail(passwordReset.email);
  }

  @Post('reset-password/apply')
  @ApiResponse({ status: 200, description: 'Email sent if account exist' })
  public async resetPasswordApply(@Body() passwordResetApplyDto: PasswordResetApplyDto) {
    return this.usersService.validatePasswordResetToken(passwordResetApplyDto.password, passwordResetApplyDto.token);
  }

  @Post('verify-exist-user')
  public async verifyUserExist(@Request() _req, @Body() email: { newMail: string }) {
    return this.usersService.verifyUserExist(email.newMail);
  }

  @Delete()
  @UseGuards(JwtAuthGuard)
  public async delete(@Req() req) {
    const user = await this.usersService.deleteOne(req.user.email);
    user.structuresLink.forEach((structureId) => {
      this.usersService.isStructureClaimed(structureId.toString()).then((userFound) => {
        if (!userFound) {
          this.structureService.deleteOne(structureId.toString());
        }
      });
    });
    return user;
  }

  @Get('dataConsentValidation')
  @UseGuards(JwtAuthGuard)
  public async dataConsentValidation(@Request() req): Promise<Structure[]> {
    return this.structureService.getAllDataConsentPendingStructures(req.user);
  }

  @UseGuards(JwtAuthGuard)
  @Get(':id')
  @ApiParam({ name: 'id', type: String, required: true })
  public async getUser(@Param() params): Promise<IUser> {
    const user = await this.usersService.findById(params.id);
    if (!user) {
      throw new HttpException('User does not exist', HttpStatus.NOT_FOUND);
    }
    return user;
  }

  @ApiResponse({ status: HttpStatus.CREATED, description: 'Description updated' })
  @ApiResponse({ status: HttpStatus.UNAUTHORIZED, description: 'Unauthorized' })
  @UseGuards(JwtAuthGuard)
  @ApiBearerAuth('JWT')
  @ApiOperation({ description: 'Updates description of a user' })
  @ApiBody({ type: DescriptionDto })
  @Post('description')
  public async updateDescription(@Req() req, @Body() body: DescriptionDto): Promise<User> {
    return this.usersService.updateDescription(req.user._id, body);
  }
}
