import { Injectable, Logger } from '@nestjs/common';
import { ElasticsearchService } from '@nestjs/elasticsearch';
import { EmployerSearchBody } from '../interfaces/employer-search-body.interface';
import { EmployerSearchResult } from '../interfaces/employer-search-response.interface';
import { Employer, EmployerDocument } from '../schemas/employer.schema';

@Injectable()
export class EmployerSearchService {
  private index = 'employer';
  private readonly logger = new Logger(EmployerSearchService.name);

  constructor(private readonly elasticsearchService: ElasticsearchService) {}

  public async indexEmployer(employer: EmployerDocument): Promise<EmployerDocument> {
    this.logger.debug(`indexEmployer ${employer.name}`);
    this.elasticsearchService.index<EmployerSearchResult, Employer>({
      index: this.index,
      id: employer._id,
      body: {
        name: employer.name,
        validated: employer.validated,
      },
    });
    return employer;
  }

  public async createEmployerIndex(): Promise<any> {
    this.logger.debug('createEmployerIndex');
    return this.elasticsearchService.indices.create({
      index: this.index,
      body: {
        settings: {
          analysis: {
            analyzer: {
              default: {
                type: 'french',
              },
              default_search: {
                type: 'french',
              },
            },
          },
        },
      },
    });
  }

  public async dropIndex(): Promise<any> {
    this.logger.debug('dropIndex');
    if (
      (
        await this.elasticsearchService.indices.exists({
          index: this.index,
        })
      ).body
    ) {
      return this.elasticsearchService.indices.delete({
        index: this.index,
      });
    }
  }

  public async deleteIndex(employer: EmployerDocument, id: string): Promise<EmployerDocument> {
    this.logger.debug('deleteIndex');
    this.elasticsearchService.delete<EmployerSearchResult, EmployerSearchBody>({
      index: this.index,
      id: id,
    });
    return employer;
  }

  public async refreshIndexStructure(): Promise<any> {
    this.logger.debug('refreshIndexStructure');
    return this.elasticsearchService.indices.refresh({
      index: this.index,
    });
  }

  public async search(searchString: string): Promise<Employer[]> {
    this.logger.debug(`search ${searchString}`);
    searchString = searchString ? searchString + '*' : '*';
    const { body } = await this.elasticsearchService.search<EmployerSearchResult>({
      index: this.index,
      body: {
        from: 0,
        size: 200,
        query: {
          query_string: {
            analyze_wildcard: 'true',
            query: searchString,
            fields: ['name'],
            fuzziness: 'AUTO',
          },
        },
      },
    });
    const sortedHits = body.hits.hits.filter(function (elem) {
      return elem._score >= body.hits.max_score / 1.5;
    });
    return sortedHits.map((item) => item._source);
  }

  public async update(employer: EmployerDocument, id: string): Promise<any> {
    this.logger.debug('update');
    return this.elasticsearchService.update({
      index: this.index,
      id: id,
      body: {
        doc: {
          name: employer.name,
          validated: employer.validated,
        },
      },
    });
  }
}
