import { HttpModule, HttpService, HttpStatus } from '@nestjs/common';
import { getModelToken } from '@nestjs/mongoose';
import { Test, TestingModule } from '@nestjs/testing';
import { AxiosResponse } from 'axios';
import { Types } from 'mongoose';
import { of } from 'rxjs';
import { ConfigurationModule } from '../../configuration/configuration.module';
import { MailerService } from '../../mailer/mailer.service';
import { CreateJobDto } from '../dto/create-job.dto';
import { JobDocument } from '../schemas/job.schema';
import { JobsService } from './jobs.service';
import { UsersService } from './users.service';

describe('JobsService', () => {
  let service: JobsService;
  let httpService: HttpService;
  let mailer: MailerService;

  const mockJobModel = {
    create: jest.fn(),
    find: jest.fn(),
    findOne: jest.fn(),
    findById: jest.fn(),
    deleteOne: jest.fn(),
  };

  const mockUserModel = {
    find: jest.fn(),
    findOne: jest.fn(),
    findById: jest.fn(),
  };
  const mockUserService = {
    replaceJobs: jest.fn(),
    getAdmins: jest.fn(),
  };

  beforeEach(async () => {
    const module: TestingModule = await Test.createTestingModule({
      imports: [ConfigurationModule, HttpModule],
      providers: [
        JobsService,
        {
          provide: getModelToken('Job'),
          useValue: mockJobModel,
        },
        {
          provide: UsersService,
          useValue: mockUserService,
        },
        {
          provide: getModelToken('User'),
          useValue: mockUserModel,
        },
        MailerService,
      ],
    }).compile();
    service = module.get<JobsService>(JobsService);
    mailer = module.get<MailerService>(MailerService);
    httpService = module.get<HttpService>(HttpService);
  });

  it('should be defined', () => {
    expect(service).toBeDefined();
  });

  describe('findAll', () => {
    it('should findAll validated jobs', async () => {
      mockJobModel.find.mockResolvedValue([
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'CNFS',
          validated: true,
          hasPersonalOffer: true,
        },
      ]);
      const reply = await service.findAll();
      expect(reply.length).toBe(1);
    });
    it('should findAll all jobs', async () => {
      mockJobModel.find.mockResolvedValue([
        {
          _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
          name: 'CNFSssss',
          validated: false,
          hasPersonalOffer: true,
        },
      ]);
      const reply = await service.findAll(false);
      expect(reply.length).toBe(1);
    });
  });

  it('findUnvalidated', async () => {
    mockJobModel.find.mockResolvedValue([
      {
        _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'CNFS',
        validated: false,
        hasPersonalOffer: true,
      },
    ]);
    const reply = await service.findAllUnvalidated();
    expect(reply[0].validated).toBeFalsy;
  });

  it('findByName', async () => {
    mockJobModel.findOne.mockResolvedValue({
      _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
      name: 'CNFS',
      validated: true,
      hasPersonalOffer: true,
    });
    const reply = await service.findByName('CNFS');
    expect(reply).toBeTruthy();
  });
  it('findOne', async () => {
    mockJobModel.findById.mockResolvedValue({
      _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
      name: 'CNFS',
      validated: true,
      hasPersonalOffer: true,
    });
    const reply = await service.findOne('6231aefe76598527c8d0b5bc');
    expect(reply).toBeTruthy();
  });

  describe('createJob', () => {
    it('create', async () => {
      mockJobModel.create.mockResolvedValue({
        _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Dev',
        validated: false,
        hasPersonalOffer: false,
      });
      const createJob: CreateJobDto = {
        name: 'Dev',
        hasPersonalOffer: false,
      };
      const reply = await service.create(createJob);
      expect(reply).toBeTruthy();
    });

    it('should send an email to admins', async () => {
      //todo fetch admin infos and check httpservice is called further
      const result: AxiosResponse = {
        data: {
          status: 200,
          content: {
            success: true,
            response: [7010],
          },
        },
        status: 200,
        statusText: 'OK',
        headers: {},
        config: {},
      };
      jest.spyOn(httpService, 'post').mockImplementationOnce(() => of(result));
      expect(await mailer.send('a@a.com', 'test', '<p>This is a test</p>')).toBe(result);
    });
  });

  it('createFromAdmin', async () => {
    mockJobModel.create.mockResolvedValue({
      _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
      name: 'Dev',
      validated: true,
      hasPersonalOffer: false,
    });
    const createJob: CreateJobDto = {
      name: 'Dev',
      hasPersonalOffer: false,
    };
    const reply = await service.create(createJob);
    expect(reply).toBeTruthy();
  });

  describe('sendAdminCreateNotification', () => {
    it('should sendAdminCreateNotification', async () => {
      const job = {
        _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'Metro',
        validated: true,
      };
      const result: AxiosResponse = {
        data: {
          status: 200,
          content: {
            success: true,
            response: [7010],
          },
        },
        status: 200,
        statusText: 'OK',
        headers: {},
        config: {},
      };
      jest.spyOn(mailer, 'send').mockResolvedValueOnce(result);
      const spyer = jest.spyOn(mailer, 'send');
      mockUserService.getAdmins.mockResolvedValueOnce([
        {
          _id: '6231aefe76598527c8d0b5bc',
          validationToken:
            'cf1c74c22cedb6b575945098db42d2f493fb759c9142c6aff7980f252886f36ee086574ee99a06bc99119079257116c959c8ec870949cebdef2b293666dbca42',
          emailVerified: true,
          email: 'admin@admin.com',
          password: '$2a$12$vLQjJ9zAWyUwiFLeQDa6w.XzrlgPBhw.2GWrjog/yuEjIaZnQwmZu',
          role: 0,
          name: 'admin',
          surname: 'admin',
          personalOffers: [],
        },
      ]);
      await service.sendAdminCreateNotification(job as JobDocument, 'adminJobCreate.ejs', 'adminJobCreate.json');
      expect(spyer.mock.calls.length).toBe(1);
    });
  });

  describe('validate', () => {
    it('should validate job', async () => {
      mockJobModel.findById.mockResolvedValueOnce({
        _id: Types.ObjectId('623aed68c5d45b6fbbaa7e60'),
        name: 'Dev',
        validated: false,
        save: jest.fn().mockResolvedValueOnce(null),
      });
      const job = await service.validate('623aed68c5d45b6fbbaa7e60');
      expect(job.validated).toBe(true);
    });
    it('should throw exception', async () => {
      mockJobModel.findById.mockResolvedValueOnce(null);
      try {
        await service.validate('623aed68c5d45b6fbbaa7e60');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Cannot validate job. It might have been already validate');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });
  });

  describe('mergeJob', () => {
    it('should delete source job', async () => {
      const reply = {
        _id: Types.ObjectId('623aed68c5d45b6fbbaa7e61'),
        name: 'Dev 2',
        validated: true,
        hasPersonalOffer: false,
      };
      mockJobModel.findById
        .mockResolvedValueOnce({
          _id: Types.ObjectId('623aed68c5d45b6fbbaa7e60'),
          name: 'Dev',
          validated: false,
          hasPersonalOffer: false,
        })
        .mockResolvedValueOnce(reply);
      jest.spyOn(service, 'deleteInvalidJob').mockRejectedValueOnce({});

      expect(
        await service.mergeJob({ sourceJobId: '623aed68c5d45b6fbbaa7e60', targetJobId: '623aed68c5d45b6fbbaa7e61' })
      ).toEqual(reply);
    });
    it('should throw error if target job is not validated', async () => {
      mockJobModel.findById
        .mockResolvedValueOnce({
          _id: Types.ObjectId('623aed68c5d45b6fbbaa7e60'),
          name: 'Dev',
          validated: false,
          hasPersonalOffer: false,
        })
        .mockResolvedValueOnce(null);
      try {
        await service.mergeJob({ sourceJobId: '623aed68c5d45b6fbbaa7e60', targetJobId: '623aed68c5d45b6fbbaa7e61' });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Cannot operate on job.');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });
    it('should throw error if no source or target job', async () => {
      mockJobModel.findById.mockResolvedValueOnce(null).mockResolvedValueOnce({
        _id: Types.ObjectId('623aed68c5d45b6fbbaa7e60'),
        name: 'Dev',
        validated: false,
        hasPersonalOffer: false,
      });
      try {
        await service.mergeJob({ sourceJobId: '623aed68c5d45b6fbbaa7e60', targetJobId: '623aed68c5d45b6fbbaa7e61' });
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Cannot operate on job.');
        expect(e.status).toBe(HttpStatus.NOT_FOUND);
      }
    });
  });

  it('deleteInvalidJob', async () => {
    mockJobModel.deleteOne.mockResolvedValueOnce(null);
    const spyer = jest.spyOn(service, 'deleteInvalidJob');
    await service.deleteInvalidJob('623aed68c5d45b6fbbaa7e60');
    expect(spyer.mock.calls.length).toEqual(1);
  });

  describe('deleteOneId', () => {
    it('should delete ', async () => {
      mockJobModel.findOne.mockResolvedValueOnce({
        _id: Types.ObjectId('623aed68c5d45b6fbbaa7e60'),
        name: 'Dev',
        validated: false,
        deleteOne: jest.fn().mockResolvedValueOnce('toto'),
      });
      const reply = await service.deleteOneId('623aed68c5d45b6fbbaa7e60');
      expect(reply).toBe('toto');
    });
    it('should delete ', async () => {
      mockJobModel.findOne.mockResolvedValueOnce(null);
      try {
        await service.deleteOneId('623aed68c5d45b6fbbaa7e60');
        expect(true).toBe(false);
      } catch (e) {
        expect(e.message).toBe('Invalid job id');
        expect(e.status).toBe(HttpStatus.BAD_REQUEST);
      }
    });
  });

  describe('update', () => {
    it('should update', async () => {
      const newJobDto = {
        name: 'CNFS',
        hasPersonalOffer: true,
      };
      const newJob = {
        _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'CNFS',
        validated: true,
        hasPersonalOffer: true,
      };
      mockJobModel.findById.mockResolvedValue({
        _id: Types.ObjectId('6231aefe76598527c8d0b5bc'),
        name: 'CNFS old',
        validated: true,
        hasPersonalOffer: false,
        save: jest.fn().mockResolvedValueOnce(newJob),
      });
      const reply = await service.update('623aed68c5d45b6fbbaa7e60', newJobDto);
      expect(reply.name).toBe('CNFS');
      expect(reply.hasPersonalOffer).toBe(true);
    });
  });

  it('should throw error', async () => {
    const newJobDto = {
      name: 'CNFS',
      hasPersonalOffer: true,
    };
    mockJobModel.findById.mockResolvedValue(null);
    try {
      await service.update('623aed68c5d45b6fbbaa7e60', newJobDto);
      expect(true).toBe(false);
    } catch (e) {
      expect(e.message).toBe('Cannot edit job. It was not found in database.');
      expect(e.status).toBe(HttpStatus.NOT_FOUND);
    }
  });
});
