import { HttpException, HttpStatus, Injectable, Logger } from '@nestjs/common';
import { InjectModel } from '@nestjs/mongoose';
import * as ejs from 'ejs';
import { Model, Query, Types } from 'mongoose';
import { MergeJobDto } from '../../admin/dto/merge-job.dto';
import { MailerService } from '../../mailer/mailer.service';
import { CreateJobDto } from '../dto/create-job.dto';
import { IJob } from '../interfaces/job.interface';
import { Job, JobDocument } from '../schemas/job.schema';
import { UsersService } from './users.service';

@Injectable()
export class JobsService {
  private readonly logger = new Logger(JobsService.name);

  constructor(
    @InjectModel(Job.name) private jobModel: Model<JobDocument>,
    private readonly userService: UsersService,
    private readonly mailerService: MailerService
  ) {}

  public async findAll(filterValidatedJobs = true): Promise<IJob[]> {
    this.logger.debug(`findAll with filterValidetedJobs: ${filterValidatedJobs}`);
    if (!filterValidatedJobs) {
      return this.jobModel.find();
    } else {
      return this.jobModel.find({ validated: true });
    }
  }

  public async findByName(name: string): Promise<JobDocument> {
    this.logger.debug('findByName');
    return this.jobModel.findOne({ name });
  }

  public async findOne(idParam: string): Promise<JobDocument> {
    this.logger.debug('findOne');
    return this.jobModel.findById(Types.ObjectId(idParam));
  }

  public async findAllUnvalidated(): Promise<IJob[]> {
    this.logger.debug('findAllUnvalidated');
    return this.jobModel.find({ validated: false });
  }

  public async create(
    job: CreateJobDto,
    validated = false,
    hasPersonalOffer = false,
    sendNotification = true
  ): Promise<Job> {
    this.logger.debug(`createJob: ${job.name} | notificationSending ${sendNotification}`);
    const result = this.jobModel
      .create({ name: job.name, validated, hasPersonalOffer })
      .then(async (postCreate: JobDocument) => {
        if (sendNotification) {
          this.sendAdminCreateNotification(
            postCreate,
            this.mailerService.config.templates.adminJobCreate.ejs,
            this.mailerService.config.templates.adminJobCreate.json
          );
        }
        return postCreate;
      });
    return result;
  }

  public async sendAdminCreateNotification(
    job: JobDocument,
    templateLocation: any,
    jsonConfigLocation: any
  ): Promise<void> {
    const config = this.mailerService.config;
    const ejsPath = this.mailerService.getTemplateLocation(templateLocation);
    const jsonConfig = this.mailerService.loadJsonConfig(jsonConfigLocation);
    const html = await ejs.renderFile(ejsPath, {
      config,
      jobName: job ? job.name : '',
    });
    const admins = await this.userService.getAdmins();
    admins.forEach((admin) => {
      this.mailerService.send(admin.email, jsonConfig.subject, html);
    });
  }

  public async validate(jobId: string): Promise<Job> {
    this.logger.debug(`validateJob: ${jobId}`);
    const job = await this.jobModel.findById(Types.ObjectId(jobId));
    if (job) {
      job.validated = true;
      job.save();
      return job;
    } else {
      throw new HttpException('Cannot validate job. It might have been already validate', HttpStatus.NOT_FOUND);
    }
  }

  public async mergeJob({ sourceJobId, targetJobId }: MergeJobDto): Promise<Job | HttpException> {
    this.logger.debug(`mergeJob: ${sourceJobId} into ${targetJobId}`);
    const sourceJob = await this.jobModel.findById(Types.ObjectId(sourceJobId));
    const targetJob = await this.jobModel.findById(Types.ObjectId(targetJobId));
    if (targetJob && sourceJob) {
      this.logger.debug(`Both jobs : ${sourceJob}, ${targetJob}`);
      this.userService.replaceJobs(sourceJob, targetJob);
      if (!sourceJob.validated) {
        this.deleteInvalidJob(sourceJobId);
      }
      return targetJob;
    } else {
      throw new HttpException('Cannot operate on job.', HttpStatus.NOT_FOUND);
    }
  }

  public async deleteInvalidJob(
    id: string
  ): Promise<
    Query<{
      ok?: number;
      n?: number;
      deletedCount?: number;
    }>
  > {
    this.logger.debug(`deleteInvalidJob: ${id}`);
    return this.jobModel.deleteOne({ _id: id });
  }

  public async deleteOneId(id: string): Promise<Job> {
    const job = await this.jobModel.findOne({ _id: id });
    if (!job) {
      throw new HttpException('Invalid job id', HttpStatus.BAD_REQUEST);
    }
    return job.deleteOne();
  }

  public async update(jobId: string, newJob: CreateJobDto): Promise<Job> {
    this.logger.debug(`editJob: ${jobId}`);
    const job = await this.jobModel.findById(Types.ObjectId(jobId));
    if (job) {
      job.name = newJob.name;
      job.hasPersonalOffer = newJob.hasPersonalOffer;
      job.save();
      return job;
    } else {
      throw new HttpException('Cannot edit job. It was not found in database.', HttpStatus.NOT_FOUND);
    }
  }
}
